
class form:
    
    def __getattr__(self, item):
            return item

    def __init__(self):
        return None
    
    def _FLOAT(self):
        return None
    
    def _STRING(self):
        return ""
    
    def _INT(self):
        return ""
    
    def _CHAR(self):
        return ""
        
    
        